// -*- C++ -*-
// $RCSfile: comprow_double.h,v $
// $Revision: 1.16.8.1 $
// $Author: langer $
// $Date: 2008-06-20 17:16:12 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

// modifications to the original SparseLib++ file are marked with "// OOF2"
#include <oofconfig.h>		// OOF2
#include "common/trace.h"
#include "common/printvec.h"
#include <list>
#include <vector>
#include <list>

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*             ********   ***                                 SparseLib++    */
/*          *******  **  ***       ***      ***               v. 1.5c        */
/*           *****      ***     ******** ********                            */
/*            *****    ***     ******** ********              R. Pozo        */
/*       **  *******  ***   **   ***      ***                 K. Remington   */
/*        ********   ********                                 A. Lumsdaine   */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*                                                                           */
/*                                                                           */
/*                     SparseLib++ : Sparse Matrix Library                   */
/*                                                                           */
/*               National Institute of Standards and Technology              */
/*                        University of Notre Dame                           */
/*              Authors: R. Pozo, K. Remington, A. Lumsdaine                 */
/*                                                                           */
/*                                 NOTICE                                    */
/*                                                                           */
/* Permission to use, copy, modify, and distribute this software and         */
/* its documentation for any purpose and without fee is hereby granted       */
/* provided that the above notice appear in all copies and supporting        */
/* documentation.                                                            */
/*                                                                           */
/* Neither the Institutions (National Institute of Standards and Technology, */
/* University of Notre Dame) nor the Authors make any representations about  */
/* the suitability of this software for any purpose.  This software is       */
/* provided ``as is'' without expressed or implied warranty.                 */
/*                                                                           */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*             Compressed row sparse matrix (0-based, Fortran)           */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef CompRow_Mat_double_H
#define CompRow_Mat_double_H


#include "vecdefs.h"
#include "engine/matvec.h"
#include "engine/matrixbase.h"
#include VECTOR_H

// class CompCol_Mat_double;
// class Coord_Mat_double;


class CompRow_Mat_double;



// The actual const_iterator class -- iterates over the nonzero
// entries of a row of the matrix.  For compatibility with the
// SparseLinkIterators, which this class emulates, the operator* must
// return a CSRRowEntry object.
class CompRow_Mat_double_const_iterator {
private:
  const CompRow_Mat_double &A;
  int row_, index_;
public:
  // Constructor to put yourself at the first element of the indicated
  // row.  
  CompRow_Mat_double_const_iterator(const CompRow_Mat_double &mtx, int row);

  // Constructor to put yourself at the start of the matrix.
  CompRow_Mat_double_const_iterator(const CompRow_Mat_double &mtx);

  ~CompRow_Mat_double_const_iterator() {}
  bool operator==(const CompRow_Mat_double_const_iterator &o) const;
  bool operator!=(const CompRow_Mat_double_const_iterator &o) const;
  bool operator<(const CompRow_Mat_double_const_iterator &o) const;
  void operator++();
  double operator*() const;
  int row() const;
  int col() const;
};




class CompRow_Mat_double : public MatrixBase {

private:
       VECTOR_double val_;       // data values (nz_ elements)
       VECTOR_int    rowptr_;    // row_ptr (dim_[0]+1 elements)
       VECTOR_int    colind_;    // col_ind  (nz_ elements)

       int base_;		// index base: offset of first element
       unsigned int nz_;	// number of nonzeros
       unsigned int dim_[2];	// number of rows, cols
  
public:
       CompRow_Mat_double(void);
       CompRow_Mat_double(const CompRow_Mat_double &S);
//        CompRow_Mat_double(const CompCol_Mat_double &C);
//        CompRow_Mat_double(const Coord_Mat_double &CO);
       CompRow_Mat_double(int M, int N, int nz, double *val, int *r, 
                               int *c, int base=0);
       CompRow_Mat_double(int M, int N, int nz, const VECTOR_double &val, 
                               const VECTOR_int &r, const VECTOR_int &c,
                               int base=0);
  virtual ~CompRow_Mat_double() {
//     std::cerr << "CompRow_Mat_double::dtor " << this << std::endl;
  }

  // Iterators added so that the submatrix extractor (below) can
  // operate on CompRow_Mat_double objects directly.
  typedef CompRow_Mat_double_const_iterator const_iterator;  
  CompRow_Mat_double_const_iterator begin(int r) const {
    return CompRow_Mat_double_const_iterator(*this, r);
  }
  CompRow_Mat_double_const_iterator end(int r) const {
    return CompRow_Mat_double_const_iterator(*this, r+1);
  }

  // construct from another matrix
  template <class MTX>
  CompRow_Mat_double(const MTX &mat);

  // construct from a submatrix of another matrix
  template <class MTX, class VEC>
  CompRow_Mat_double(const MTX &mat, const VEC &rowmap, const VEC &colmap);

  typedef double value_type;

    
/*******************************/
/*  Access and info functions  */
/*******************************/

       double&      val(int i) { return val_(i); }
       int&         row_ptr(int i) { return rowptr_(i); }
       int&         col_ind(int i) { return colind_(i);}

       const double&      val(int i) const { return val_(i); }
       const int&         row_ptr(int i) const { return rowptr_(i); }
       const int&         col_ind(int i) const { return colind_(i);}

       unsigned int          dim(int i) const {return dim_[i];};
       unsigned int          size(int i) const {return dim_[i];};
       unsigned int          NumNonzeros() const {return nz_;};
       int          base() const {return base_;}
  int nrows() const { return dim_[0]; }	// OOF2
  int ncols() const { return dim_[1]; }	// OOF2
  int nnz() const { return nz_; } // OOF2

/*******************************/
/*  Assignment  operator       */
/*******************************/

       CompRow_Mat_double& operator=(const CompRow_Mat_double &R);
       CompRow_Mat_double& newsize(int M, int N, int nz);

/***********************************/
/*  General access function (slow) */
/***********************************/

  double       operator() (int i, int j) const;        
  double&      set(int i, int j);
  
/***********************************/
/*  Matrix/Vector multiply         */
/***********************************/

  virtual VECTOR_double operator*(const VECTOR_double &x) const;
  VECTOR_double trans_mult(const VECTOR_double &x) const;
  virtual CompRow_Mat_double &operator*=(double);
  void clear();
  CompRow_Mat_double * add(const CompRow_Mat_double &) const; //OOF2
};

CompRow_Mat_double operator*(const CompRow_Mat_double&, double);
CompRow_Mat_double operator*(double, const CompRow_Mat_double&);

std::ostream& operator << (std::ostream & os, const CompRow_Mat_double & mat);


//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//
//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//
//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// templated constructors added for OOF2 

// Extract a submatrix from mat. rowmap[i] is the row of the submatrix
// corresponding to row i of mat.  If rowmap[i] == -1, then row i
// should not be included in the submatrix.  If rowmap[i] ==
// rowmap[j], then rows i and j of mat are added together in the
// submatrix.  Likewise for columns.

// static void comprowdebug(std::ostream &os,
// 			 int ii, int jj, int i, int j, double v)
// {
//   if(i != -1 && j != -1)
//     os << "CompRow: (" << ii << "," << jj
//        << ")->(" << i << "," << j << ")  val=" << v << std::endl;
// }

class CSRRowEntry {
private:
  int col_;
  double val_;
public:
  CSRRowEntry() : col_(0), val_(0) {}
  CSRRowEntry(int i, double x) : col_(i), val_(x) {}
  void operator+=(double x) { val_ += x; }
  int col() const { return col_; }
  double val() const { return val_; }
};

template <class MTX, class VEC>
CompRow_Mat_double::CompRow_Mat_double(const MTX &mat,
				       const VEC &rowmap, const VEC &colmap)
  : base_(0)
{
  // Count how many rows and columns are in the target matrix
  int nr = 0;
  for(typename VEC::size_type i=0; i<rowmap.size(); i++) {
    if(rowmap[i] > nr) nr = rowmap[i];
  }
  nr++;
  dim_[0] = nr;

  int nc = 0;
  for(typename VEC::size_type i=0; i<colmap.size(); i++) {
    if(colmap[i] > nc) nc = colmap[i];
  }
  nc++;
  dim_[1] = nc;

  // Invert the rowmap to find out which rows of the source matrix
  // contribute to each row of the target.
  std::vector<std::list<int> > invrowmap(nr);
  for(typename VEC::size_type i=0; i<rowmap.size(); i++) {
    if(rowmap[i] > -1) {
      invrowmap[rowmap[i]].push_front(i);
    }
  }

  rowptr_.newsize(nr + 1);

  int nz = 0;			// non-zeros found so far
  rowptr_[0] = 0;

  // loop over rows of the target matrix
  for(int i=0; i<nr; i++) {
    // make a list of which columns appear in this row
    std::list<CSRRowEntry> cols;	// doubly linked list
    // loop over source rows that contribute to this target row
    for(std::list<int>::const_iterator isrc=invrowmap[i].begin();
	isrc != invrowmap[i].end(); ++isrc)
      {
	// loop over entries in source row
	int srcrow = *isrc;
	for(typename MTX::const_iterator ij=mat.begin(srcrow);
	    ij < mat.end(srcrow); ++ij)
	  {
	    int j = colmap[ij.col()];
	    if(j >= 0) {	// is this column being used?
	      // has this column already been found?
	      bool found = false;
	      for(std::list<CSRRowEntry>::iterator tj=cols.begin();
		  tj != cols.end() && !found; ++tj)
		{
		  int targetcol = (*tj).col();
		  if(j == targetcol) {
		    (*tj) += *ij; // add to preexisting entry
		    found = true;
		  }
		  else if(j < targetcol) {
		    cols.insert(tj, CSRRowEntry(j, *ij)); // insert before tj
		    found = true;
		  }
		} // end loop over target columns
	      if(!found) {
		cols.push_back(CSRRowEntry(j, *ij));	// insert at end
	      }
	    } // this source column is used
	  } // loop over entries in source row
      }	// loop over source rows that contribute to this target row

    // copy data into target matrix
    nz += cols.size();
    rowptr_[i+1] = nz;		// beginning of next row
    for(std::list<CSRRowEntry>::const_iterator tj=cols.begin();
	tj != cols.end(); ++tj)
      {
	val_.push_back((*tj).val());
	colind_.push_back((*tj).col());
      }

  } // end i-loop over target rows
  nz_=nz;
}

// Construct from another type of matrix, without extracting a submatrix
template <class MTX>
CompRow_Mat_double::CompRow_Mat_double(const MTX &mat)
  : base_(0)
{
  dim_[0] = mat.nrows();
  dim_[1] = mat.ncols();
  // count non-zero entries
  nz_ = 0;
  for(typename MTX::const_iterator ij = mat.begin(); ij < mat.end(); ++ij) {
    nz_++;
  }
  rowptr_.newsize(dim_[0] + 1);
  val_.newsize(nz_);
  colind_.newsize(nz_);

  int lastrow = -1;
  int nz = 0;
  for(typename MTX::const_iterator ij = mat.begin(); ij < mat.end(); ++ij) {
    int i = ij.row();
    int j = ij.col();
    if(i != lastrow) {
      for(int rr = lastrow+1; rr<=i; rr++) {
	rowptr_[rr] = nz;
      }
      lastrow = i;
    }
    val_[nz] = *ij;
    colind_[nz] = j;
    nz++;
    rowptr_[i+1] = nz; // beginning of the *next* row,
    // if there are no more entries in *this* row
  }
  // All remaining rows are empty, so their starting and ending
  // indices are all set to nz
  for(unsigned int rr=lastrow+1; rr<=dim_[0]; rr++)
    rowptr_[rr] = nz;
}

#endif  
/* CompRow_Mat_double_H */

