// -*- C++ -*-
// $RCSfile: oofimage3d.C,v $
// $Revision: 1.2 $
// $Author: vrc $
// $Date: 2007-09-26 22:03:27 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include <Python.h>
#include "image/oofimage3d.h"
#ifdef HAVE_MPI
#include "common/mpitools.h"
#endif ////HAVE_MPI
#include "common/IO/bitoverlay.h"
//#include "common/IO/stringimage.h"
#include "common/ooferror.h"
//#include "common/boolarray.h"
//#include "common/doublearray.h"
#include <math.h>
#include <set>
//#include <Magick++.h>
// TODO: include these in a portable way!"
#include "vtk-5.0/vtkImageReader2.h"
#include "vtk-5.0/vtkPNMReader.h"
#include "vtk-5.0/vtkJPEGReader.h"
#include "vtk-5.0/vtkTIFFReader.h"
#include "vtk-5.0/vtkImageConstantPad.h"
#include "vtk-5.0/vtkImageChangeInformation.h"
#include "vtk-5.0/vtkImageExtractComponents.h"
#include "vtk-5.0/vtkImageAppendComponents.h"
#include "vtk-5.0/vtkImageGaussianSmooth.h"
#include "vtk-5.0/vtkImageLuminance.h"
#include "vtk-5.0/vtkImageFlip.h"
#include "vtk-5.0/vtkPNMWriter.h"
#include "vtk-5.0/vtkMetaImageWriter.h"
#include "vtk-5.0/vtkImageMathematics.h"
#include "vtk-5.0/vtkImageMedian3D.h"
#include "vtk-5.0/vtkImageThreshold.h"
#include "vtk-5.0/vtkImageCast.h"
#include "vtk-5.0/vtkPicker.h"
#include "vtk-5.0/vtkVolume.h"
#include "vtk-5.0/vtkFixedPointVolumeRayCastMapper.h"
#include "vtk-5.0/vtkVolumeProperty.h"
#include "vtk-5.0/vtkPiecewiseFunction.h"
#include "vtk-5.0/vtkVolumeProperty.h"
#include "vtk-5.0/vtkColorTransferFunction.h"
#include "vtk-5.0/vtkDataSet.h"
#include "vtk-5.0/vtkPointData.h"
//#include "vtk-5.0/vtk.h"


// get rid of this after debugging
#include <iostream>

OOFImage3D::OOFImage3D(const std::string &name, const std::string &format, const std::string &filepattern, int numfiles)
  : name_(name)
{

	vtkImageReader2 *imageReader;
	vtkIndent indent;
	int depth = numfiles;
	// the width and height seem to be set automatically
	int extent[6] = {0,0,0,0,0,depth-1};

// 	// TODO: add more formats, handle exceptions
	if(format.compare("pnm")==0 || format.compare("pbm")==0 || format.compare("ppm")==0) {
		imageReader = vtkPNMReader::New();
	}
	if(format.compare("jpeg")==0) {
		imageReader = vtkJPEGReader::New();
	}
	if(format.compare("tiff")==0) {
		imageReader = vtkTIFFReader::New();
	}
	
 	imageReader->SetDataExtent(extent);
	imageReader->SetDataSpacing(1,1,1);
	imageReader->SetFilePattern( filepattern.c_str() );

	image = imageReader->GetOutput();
	image->Update();


	// TODO: manipulate channels for b/w images?
// 	vtkImageData *alpha;
// 	alpha = makeAlphaChannel();
// 	vtkImageData *rgb;
// 	rgb = getRGB();
// 	combineChannels(rgb, alpha);

	image = getRGB();
	padImage(1);

	setup();

}

OOFImage3D::OOFImage3D(const std::string &name)
  : name_(name)
{
}

void OOFImage3D::setSize(const Coord *sighs) {
  size_ = *sighs;
	if(sizeInPixels_ != size_){
		// We need to change the data spacing of the image object
		vtkImageChangeInformation *changer;
		double x_spacing, y_spacing, z_spacing;

		x_spacing = size_(0)/(double)sizeInPixels_(0);
		y_spacing = size_(1)/(double)sizeInPixels_(1);
		z_spacing = size_(2)/(double)sizeInPixels_(2);
		changer = vtkImageChangeInformation::New();
		changer->SetInputConnection(image->GetProducerPort());
		// TODO: check that all spacings are non-zero and throw error.
		if (!(x_spacing && y_spacing && z_spacing)) { cout << "zero spacing" << endl;}
		changer->SetOutputSpacing(x_spacing, y_spacing, z_spacing);
		image = changer->GetOutput();
		image->Update();
	}	


}

void OOFImage3D::setup() {
	// TODO: catch errors?
	int *sighs = image->GetDimensions();
	// subtract 1 because of padding
	sizeInPixels_ = ICoord(sighs[0]-1, sighs[1]-1, sighs[2]-1);
}

OOFImage3D::~OOFImage3D() 
{
	// would this delete the whole pipeline?
// 	cout << "deleting " << this << endl;
// 	image->Delete();
}

// This is used by the image modifiers so that we may have a ring
// buffer.  One way to do this is to just doing a shallow copy on the
// image pointer.  The idea is that if we undo, we go back to an
// earlier point in the pipeline.  The bad thing is that the filters
// created past that point won't be deleted explicitly and if the copy
// image is deleted, the shared pipeline will be deleted.  OOF2 uses
// the image magick copy constructor, which presumably does a deep
// copy, so that's what we'll do here.  Then we can destroy that copy
// without worrying about the pipeline we have here...
OOFImage3D *OOFImage3D::clone(const std::string &nm) const {
  OOFImage3D *copy = new OOFImage3D(nm);
	copy->image = vtkImageData::New();
	copy->image->DeepCopy(image);
  copy->setup();	 
	copy->size_ = size_;
  copy->setMicrostructure(microstructure);
  return copy;
}

// For now, save just writes the slices as pnms.  We
// might want to implement different formats later, but for now, this
// is a sensible default since it's something we can read in easily.
void OOFImage3D::save(const std::string &filepattern) {
	vtkPNMWriter *writer;
	writer = vtkPNMWriter::New();

	padImage(-1);
	vtkImageData *rgb = getRGB();

	writer->SetFilePattern(filepattern.c_str());
	writer->SetInputConnection(rgb->GetProducerPort());
	writer->Update();
	writer->Write();

}

const CColor OOFImage3D::operator[](const ICoord &c) const {
	return CColor(image->GetScalarComponentAsFloat(c(0),c(1),c(2),0)/255,
								image->GetScalarComponentAsFloat(c(0),c(1),c(2),1)/255,
								image->GetScalarComponentAsFloat(c(0),c(1),c(2),2)/255);

}

bool OOFImage3D::compare(const OOFImage3D &other, double tol) const {
  if (sizeInPixels_ != other.sizeInPixels_) return false;

  for(int i=0;i<sizeInPixels_(0);i++) {
    for(int j=0;j<sizeInPixels_(1);j++) {
			for(int k=0;j<sizeInPixels_(2);j++) {
				CColor c0 = (*this)[ICoord(i,j,k)];
				CColor c1 = other[ICoord(i,j,k)];
				if (!c0.compare(c1, tol))
					return false;
			}
    }
  }
  return true;

}

void OOFImage3D::padImage(int amount) {
	// pad the image with extra voxels because of vtk off by 1
  // annoyance in calculating image bounds
	image->Update();
	int *currentextent = image->GetExtent();
	int newextent[6] = {0,0,0,0,0,0};
	vtkImageConstantPad *padder;

	// Be careful to not overwrite the extent in the current image.	
	for(int i = 1; i<7; i+=2) newextent[i]=currentextent[i]+amount;

	padder = vtkImageConstantPad::New();
	padder->SetInputConnection(image->GetProducerPort());
	padder->SetOutputWholeExtent(newextent);
	padder->SetOutputNumberOfScalarComponents(4);
	padder->SetConstant(0); 

	image = padder->GetOutput();
	image->Update();	

}	


// Not sure we'll need these functions...
// vtkImageData* OOFImage3D::makeAlphaChannel() {
// 	vtkImageExtractComponents *extractor;
// 	vtkImageData *alpha;
// 	int *sighs;

// 	extractor = vtkImageExtractComponents::New();
// 	extractor->SetInputConnection(image->GetProducerPort());
// 	extractor->SetComponents(0);
// 	alpha = extractor->GetOutput();
// 	alpha->Update();
// 	sighs = alpha->GetDimensions();
// 	// TODO: more efficient way to do this?
// 	for(int i = 0; i<sighs[0]; i++) {
// 		for(int j = 0; j<sighs[1]; j++) {
// 			for(int k = 0; k<sighs[2]; k++) {
// 				alpha->SetScalarComponentFromFloat(i,j,k,0,0.0);
// 			}
// 		}
// 	}
// 	return alpha;
// }

void OOFImage3D::combineChannels(vtkImageData *color, vtkImageData *alpha){
	vtkImageAppendComponents *appender;
	appender = vtkImageAppendComponents::New();
	int numcomponents = color->GetNumberOfScalarComponents();
	if(numcomponents==1)
		for(int i = 0; i<3; i++) appender->AddInputConnection(color->GetProducerPort());
	if(numcomponents==3)
		appender->AddInputConnection(color->GetProducerPort());
	appender->AddInputConnection(alpha->GetProducerPort());
	image = appender->GetOutput();
	image->Update();

}

vtkImageData* OOFImage3D::getRGB() {
	// get a 3 channel RGB image from image of 1,2,3,or 4 channels
	vtkImageAppendComponents *appender;
	appender = vtkImageAppendComponents::New();
	vtkImageExtractComponents *extractor;
	extractor = vtkImageExtractComponents::New();
	vtkImageData *rgb;
	int numcomponents = image->GetNumberOfScalarComponents();

	switch(numcomponents) {
	case 1:
		for(int i = 0; i<3; i++) appender->AddInput(image);
		rgb = appender->GetOutput();
		break;
	case 2:
		extractor->SetInputConnection(image->GetProducerPort());
		extractor->SetComponents(0);
		for(int i = 0; i<3; i++) appender->AddInput(extractor->GetOutput());
		rgb = appender->GetOutput();
		break;
	case 3:
		rgb = image;
		break;
	case 4:
		extractor->SetInputConnection(image->GetProducerPort());
		extractor->SetComponents(0,1,2);
		rgb = extractor->GetOutput();
		//default --- throw error
	}
	rgb->Update();
	return rgb;

}

vtkImageData* OOFImage3D::getAlpha() {
	vtkImageExtractComponents *extractor;
	extractor = vtkImageExtractComponents::New();
	vtkImageData *alpha;
	
	extractor->SetInputConnection(image->GetProducerPort());
	extractor->SetComponents(3);
	alpha = extractor->GetOutput();

	alpha->Update();
	return alpha;
}


PyObject* OOFImage3D::getVolumeActor() {
	vtkFixedPointVolumeRayCastMapper *mapper;
	mapper = vtkFixedPointVolumeRayCastMapper::New();
	mapper->IntermixIntersectingGeometryOn();
	vtkIndent indent;
	image->Update();
	mapper->SetInputConnection(image->GetProducerPort());

	vtkVolumeProperty *volproperty;
	volproperty = vtkVolumeProperty::New();

	vtkColorTransferFunction *red;
	red = vtkColorTransferFunction::New();
	red->AddRGBSegment(0,0,0,0,255,1,0,0);
	volproperty->SetColor(0,red);
	vtkColorTransferFunction *green;
	green = vtkColorTransferFunction::New();
	green->AddRGBSegment(0,0,0,0,255,0,1,0);
	volproperty->SetColor(1,green);
	vtkColorTransferFunction *blue;
	blue = vtkColorTransferFunction::New();
	blue->AddRGBSegment(0,0,0,0,255,0,0,1);
	volproperty->SetColor(2,blue);
	vtkPiecewiseFunction *opacity;
	opacity = vtkPiecewiseFunction::New();
	opacity->AddSegment(0,1,255,0);
	volproperty->SetScalarOpacity(3,opacity);

	vtkVolume *volume;
	volume = vtkVolume::New();
	volume->SetMapper(mapper);
	volume->SetProperty(volproperty);

	return vtkPythonGetObjectFromPointer(volume);
}

// void OOFImage3D::addImageToPicker(PyObject* pypicker){
// 	vtkPicker *picker;
// 	picker = vtkPythonGetPointerFromObject(pypicker);
// }

void OOFImage3D::imageChanged() {
  ++timestamp;			// marks image as changed
}

void OOFImage3D::gray() {
	vtkImageData *rgb = getRGB();
	vtkImageData *alpha = getAlpha();
	vtkImageData *gray;
	vtkImageLuminance *luminance;

	luminance = vtkImageLuminance::New();

	luminance->SetInputConnection(rgb->GetProducerPort());
	gray = luminance->GetOutput();
	gray->Update();

	combineChannels(gray,alpha);
	imageChanged();

}

void OOFImage3D::threshold(double T) {
	vtkImageData *rgb = getRGB();
	vtkImageData *alpha = getAlpha();
	vtkImageData *gray;
	vtkImageLuminance *luminance;
	vtkImageThreshold *thold1, *thold2; 

	luminance = vtkImageLuminance::New();

	luminance->SetInputConnection(rgb->GetProducerPort());
	gray = luminance->GetOutput();
	
	thold1 = vtkImageThreshold::New();
	thold1->ThresholdByUpper(T*255);
	thold1->SetOutValue(0);
	thold1->SetInputConnection(gray->GetProducerPort());
	
	thold2 = vtkImageThreshold::New();
	thold2->ThresholdByLower(T*255);
	thold2->SetOutValue(255);
	thold2->SetInputConnection(thold1->GetOutputPort());

	gray = thold2->GetOutput();
	gray->Update();

	combineChannels(gray,alpha);
	imageChanged();

}

void OOFImage3D::blur(double radius, double sigma) {

	vtkImageGaussianSmooth *gauss;

	padImage(-1);

	gauss = vtkImageGaussianSmooth::New();
	gauss->SetRadiusFactors(radius, radius, radius);
	gauss->SetStandardDeviation(sigma, sigma, sigma);
	gauss->SetInputConnection(image->GetProducerPort());

	image = gauss->GetOutput();
	
	padImage(1);
	
	imageChanged();

}


void OOFImage3D::dim(double factor) {
	vtkImageData *rgb = getRGB();
	vtkImageData *alpha = getAlpha();
	vtkImageMathematics *dimmer;

	dimmer = vtkImageMathematics::New();
	dimmer->SetOperationToMultiplyByK();
	dimmer->SetConstantK(factor);
	dimmer->SetInputConnection(rgb->GetProducerPort());
	rgb = dimmer->GetOutput();
	rgb->SetScalarTypeToUnsignedChar();
	rgb->Update();
	
	combineChannels(rgb,alpha);
	imageChanged();

}

void OOFImage3D::fade(double factor) {
	vtkImageData *rgb = getRGB();
	vtkImageData *alpha = getAlpha();
	vtkImageMathematics *fade1, *fade2;

	fade1 = vtkImageMathematics::New();
	fade1->SetOperationToMultiplyByK();
	fade1->SetConstantK(1.0-factor);
	fade1->SetInputConnection(rgb->GetProducerPort());
	
	fade2 = vtkImageMathematics::New();
	fade2->SetOperationToAddConstant();
	fade2->SetConstantC(255*factor);
	fade2->SetInputConnection(fade1->GetOutputPort());
	
	rgb = fade2->GetOutput();
	rgb->SetScalarTypeToUnsignedChar();
	rgb->Update();
	
	combineChannels(rgb,alpha);
	imageChanged();

}

void OOFImage3D::negate(double dummy) {
	vtkImageData *rgb = getRGB();
	vtkImageData *alpha = getAlpha();
	vtkImageMathematics *negator, *corrector;

	negator = vtkImageMathematics::New();
	negator->SetOperationToMultiplyByK();
	negator->SetConstantK(-1.0);
	negator->SetInputConnection(rgb->GetProducerPort());

	corrector = vtkImageMathematics::New();
	corrector->SetOperationToAddConstant();
	corrector->SetConstantC(255);
	corrector->SetInputConnection(negator->GetOutputPort());

	rgb = corrector->GetOutput();
	rgb->SetScalarTypeToUnsignedChar();
	rgb->Update();
	
	combineChannels(rgb,alpha);
	imageChanged();

}

void OOFImage3D::medianFilter(int radius) {
	padImage(-1);
	vtkImageData *rgb = getRGB();
	vtkImageData *alpha = getAlpha();
	vtkImageMedian3D *median;
	
	median = vtkImageMedian3D::New();
	median->SetKernelSize(radius*2, radius*2, radius*2);
	median->SetInputConnection(rgb->GetProducerPort());
	
	rgb = median->GetOutput();
	rgb->Update();
	
	combineChannels(rgb,alpha);
	padImage(1);
	imageChanged();
}

void OOFImage3D::normalize() {
	padImage(-1);
	vtkImageData *alpha = getAlpha();
	
	vtkImageAppendComponents *appender;
	vtkImageExtractComponents *extractor[3];
	vtkImageMathematics *adder[3], *mult[3];
	double *minmax, K;

	appender = vtkImageAppendComponents::New();

	for(int i=0; i<3; i++) {

		extractor[i] = vtkImageExtractComponents::New();
		extractor[i]->SetInputConnection(image->GetProducerPort());
		extractor[i]->SetComponents(i);
		extractor[i]->Update();
		
		minmax = extractor[i]->GetOutput()->GetScalarRange();
		
		adder[i] = vtkImageMathematics::New();
		adder[i]->SetInputConnection(extractor[i]->GetOutputPort());
		adder[i]->SetOperationToAddConstant();
		if(minmax[0] > 0 && minmax[0] != minmax[1]) {
			adder[i]->SetConstantC(256-minmax[0]);
		}
		else{
			adder[i]->SetConstantC(0);
		}

		mult[i] = vtkImageMathematics::New();
		mult[i]->SetInputConnection(adder[i]->GetOutputPort());
		mult[i]->SetOperationToMultiplyByK();
		if(minmax[0]!=minmax[1]) {
			K = 255.0/(minmax[1]-minmax[0]);
			mult[i]->SetConstantK(K);
		}
		else {
			mult[i]->SetConstantK(1.0);
		}
		mult[i]->GetOutput()->SetScalarTypeToUnsignedChar();


 		appender->AddInputConnection(mult[i]->GetOutputPort());

	}

	appender->AddInputConnection(alpha->GetProducerPort());
	image = appender->GetOutput();
	padImage(1);
	imageChanged();

}
	
void OOFImage3D::flip(const std::string &axis) {

	padImage(-1);
	vtkImageFlip *flipper[3];
	
	for(int i=0; i<axis.size(); i++) {
		flipper[i] = vtkImageFlip::New();
		flipper[i]->SetFilteredAxis((int)axis[i]-(int)'x');
		flipper[i]->SetInputConnection(image->GetProducerPort());
		image = flipper[i]->GetOutput();
	}

	image->Update();
	padImage(1);
	imageChanged();

}


// This is kind of slow for very large images.  It might make sense to
// have a progress bar.
void OOFImage3D::getColorPoints(const CColor &ref, 
			      const ColorDifference &diff,
			      BoolArray &selected)
const
{
//   for(ConstOOFImage3DIterator i=this->begin(); i!=this->end(); ++i)
//     if (diff.contains(ref, *i)) selected[i.coord()] = true;

	int i,j,k;
	ICoord coord;
	for (i=0; i<sizeInPixels_(0); i++) {
		for (j=0; j<sizeInPixels_(1); j++) {
			for (k=0; k<sizeInPixels_(2); k++) {
				coord = ICoord(i,j,k);
				if (diff.contains(ref, operator[](coord))) selected[coord] = true;
			}
		}
	}

}





//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// OOFImage3D::iterator OOFImage3D::begin() {
//   return OOFImage3DIterator(*this, 0);
// }

// OOFImage3D::const_iterator OOFImage3D::begin() const {
//   return ConstOOFImage3DIterator(*this, 0);
// }

// OOFImage3D::iterator OOFImage3D::end() {
//   return OOFImage3DIterator(*this, sizeInPixels_(0)*sizeInPixels_(1)*sizeInPixels_(2));
// }

// OOFImage3D::const_iterator OOFImage3D::end() const {
//   return ConstOOFImage3DIterator(*this, sizeInPixels_(0)*sizeInPixels_(1)*sizeInPixels_(2));
// }


// ICoord OOFImage3DIterator::coord() const {
//   int width = image.sizeInPixels()(0);
//   int y = pos/width;
//   int x = pos - width*y;
//   return ICoord(x, y, 0);
// }

// ICoord ConstOOFImage3DIterator::coord() const {
//   int width = image.sizeInPixels()(0);
//   int y = pos/width;
//   int x = pos - width*y;
//   return ICoord(x, y, 0);
// }

// bool operator==(const OOFImage3DIterator &a, const OOFImage3DIterator &b) {
//   return a.pos == b.pos;
// }

// bool operator!=(const OOFImage3DIterator &a, const OOFImage3DIterator &b) {
//   return a.pos != b.pos;
// }


// bool operator==(const ConstOOFImage3DIterator &a, const ConstOOFImage3DIterator &b){
//   return a.pos == b.pos;
// }

// bool operator!=(const ConstOOFImage3DIterator &a, const ConstOOFImage3DIterator &b){
//   return a.pos != b.pos;
// }

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//
